use crate::model::{
    boat::Boat,
    location::Location,
    user::{AdminUser, User},
};
use rocket::{
    form::Form,
    get, post,
    request::FlashMessage,
    response::{Flash, Redirect},
    routes, FromForm, Route, State,
};
use rocket_dyn_templates::{tera::Context, Template};
use sqlx::SqlitePool;

#[get("/boat")]
async fn index(
    db: &State<SqlitePool>,
    admin: AdminUser,
    flash: Option<FlashMessage<'_>>,
) -> Template {
    let boats = Boat::all(db).await;
    let locations = Location::all(db).await;
    let users = User::all(db).await;

    let mut context = Context::new();
    if let Some(msg) = flash {
        context.insert("flash", &msg.into_inner());
    }
    context.insert("boats", &boats);
    context.insert("locations", &locations);
    context.insert("users", &users);
    context.insert("loggedin_user", &admin.user);

    Template::render("admin/boat/index", context.into_json())
}

#[get("/boat/<boat>/delete")]
async fn delete(db: &State<SqlitePool>, _admin: AdminUser, boat: i32) -> Flash<Redirect> {
    let boat = Boat::find_by_id(db, boat).await;
    match boat {
        Some(boat) => {
            boat.delete(db).await;
            Flash::success(
                Redirect::to("/admin/boat"),
                format!("Sucessfully deleted boat {}", boat.name),
            )
        }
        None => Flash::error(Redirect::to("/admin/boat"), "Boat does not exist"),
    }
}

#[derive(FromForm)]
struct BoatEditForm<'r> {
    id: i32,
    name: &'r str,
    amount_seats: i64,
    year_built: Option<i64>,
    boatbuilder: Option<&'r str>,
    default_shipmaster_only_steering: bool,
    skull: bool,
    external: bool,
    location_id: Option<i64>,
    owner: Option<i64>,
}

#[post("/boat", data = "<data>")]
async fn update(
    db: &State<SqlitePool>,
    data: Form<BoatEditForm<'_>>,
    _admin: AdminUser,
) -> Flash<Redirect> {
    let boat = Boat::find_by_id(db, data.id).await;
    let Some(boat) = boat else {
            return Flash::error(
                Redirect::to("/admin/boat"),
                format!("Boat with ID {} does not exist!", data.id),
            )
    };

    if !boat
        .update(
            db,
            data.name,
            data.amount_seats,
            data.year_built,
            data.boatbuilder,
            data.default_shipmaster_only_steering,
            data.skull,
            data.external,
            data.location_id,
            data.owner,
        )
        .await
    {
        return Flash::error(
            Redirect::to("/admin/boat"),
            format!("Boat with ID {} could not be updated!", data.id),
        );
    }

    Flash::success(Redirect::to("/admin/boat"), "Successfully updated boat")
}

#[derive(FromForm)]
struct BoatAddForm<'r> {
    name: &'r str,
    amount_seats: i64,
    year_built: Option<i64>,
    boatbuilder: Option<&'r str>,
    default_shipmaster_only_steering: bool,
    skull: bool,
    external: bool,
}

#[post("/boat/new", data = "<data>")]
async fn create(
    db: &State<SqlitePool>,
    data: Form<BoatAddForm<'_>>,
    _admin: AdminUser,
) -> Flash<Redirect> {
    if Boat::create(
        db,
        data.name,
        data.amount_seats,
        data.year_built,
        data.boatbuilder,
        data.default_shipmaster_only_steering,
        data.skull,
        data.external,
    )
    .await
    {
        Flash::success(Redirect::to("/admin/boat"), "Successfully created boat")
    } else {
        Flash::error(
            Redirect::to("/admin/boat"),
            format!("Error while creating boat {} in DB", data.name),
        )
    }
}

pub fn routes() -> Vec<Route> {
    routes![index, create, delete, update]
}
